<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/api_csv.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["plugins"]."unit_testing/ut/api.php");
 require_once($_path["format"]."csv.php");
 require_once($_path["api"]."csv.php");

 class UT_API_CSV extends UT_API
 {
  function Constructor()
  {
   global $_path;

   $object=new API_CSV();

   if(!is_object($object)||get_class($object)!="API_CSV")
   {
    $this->e->add("Constructor doesn't return a API_CSV object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->type!="CSV")
   {
    $this->e->add("API type isn't CSV");
    return UT_FAILED;
   }

   if($object->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($object->encoding!="ISO-8859-1")
   {
    $this->e->add("API enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->contentType!="text/csv")
   {
    $this->e->add("API content type isn't text/csv");
    return UT_FAILED;
   }

   if($object->filename)
   {
    $this->e->add("API filename shouldn't be set");
    return UT_FAILED;
   }

   if(!isset($object->config))
   {
    $this->e->add("API configuration not set");
    return UT_FAILED;
   }

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   //merge config
   $api=new API_CSV("server", $this->_config(true)); //API enabled

   if(!$api->config["csv"])
   {
    $this->e->add("API CSV should be enabled");
    return UT_FAILED;
   }

   $api=new API_CSV("server", $this->_config(false)); //API disabled

   if($api->config["csv"])
   {
    $this->e->add("API CSV should be disabled");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new API_CSV();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("API_CSV object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->mode="different";
   if($orig->mode==$copy->mode)
   {
    $this->e->add("API_CSV object copy also modified original \"mode\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("API_CSV object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_API::Copy_Constructor(); //run parent Copy_Constructor tests
  }
 }


 class UT_API_CSV_Server extends UT_API_CSV
 {
  function Constructor()
  {
   $object=new API_CSV("server");

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Error()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("server");

   if(!$api->e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   if(!$api->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   if(!$api->error())
   {
    $this->e->add("API error should be successful");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)!=2)
   {
    $this->e->add("Response should contain 2 rows");
    return UT_FAILED;
   }

   if(sizeof($csv->data[0])!=3)
   {
    $this->e->add("Response should contain 3 columns");
    return UT_FAILED;
   }

   if($csv->data[0][0]!="errors")
   {
    $this->e->add("Error respond error doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][1]!="Level")
   {
    $this->e->add("Error respond level doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][2]!="Extra")
   {
    $this->e->add("Error respond extra doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][0]!="Error")
   {
    $this->e->add("Error respond error doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][1]!="error")
   {
    $this->e->add("Error respond level doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][2]!="Text")
   {
    $this->e->add("Error respond extra doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Version()
  {
   $api=new API_CSV("server", $this->_config(true));

   //no header version
   $api->e->clear();
   $response='fail';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail request")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail request");
    return UT_FAILED;
   }


   //valid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-1.0",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail request")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail request");
    return UT_FAILED;
   }


   //invalid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if($api->getHeaderVersion()!="0.1a")
   {
    $this->e->add("Requested API version isn't 0.1a");
    return UT_FAILED;
   }

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="0.1a")
   {
    $this->e->add("API version isn't 0.1a");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported version: \"0.1a\"")===false)
   {
    $this->e->add("First error should contain: unsupported version: \"0.1a\"");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Data_Request()
  {
   $api=new API_CSV("server", $this->_config(true));

   $request="data;files;view;all
nr;file;fileData
123;JAS & API;ZnBnbcZwOSk=
";

   if(!$api->parse($request))
   {
    $this->e->add("Data request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Data parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="data")
   {
    $this->e->add("Data parse should set data command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["data"])||get_class($api->commands[0]["data"])!="Data")
   {
    $this->e->add("Data parse no data object");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->module!="files")
   {
    $this->e->add("Data parse module incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->action!="view")
   {
    $this->e->add("Data parse action incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->extra!="all")
   {
    $this->e->add("Data parse extra incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["count"])
   {
    $this->e->add("Data parse count incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("nr")!=123)
   {
    $this->e->add("Data parse getValue nr incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("file")!="JAS & API")
   {
    $this->e->add("Data parse getValue name incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("fileData")!="fpgmp9)")
   {
    $this->e->add("Data parse getValue name incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Language_Request()
  {
   $api=new API_CSV("server", $this->_config(true));

   $request='"language";"modules";"nl"';

   if(!$api->parse($request))
   {
    $this->e->add("Language request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Language parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="language")
   {
    $this->e->add("Language parse should set language command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["language"])||get_class($api->commands[0]["language"])!="Language")
   {
    $this->e->add("Language parse no language object");
    return UT_FAILED;
   }

   if($api->commands[0]["lang"]!="nl")
   {
    $this->e->add("Data parse lang incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["set"]!="modules")
   {
    $this->e->add("Data parse set incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Modules_Request()
  {
   $api=new API_CSV("server", $this->_config(true));

   $request='modules';

   if(!$api->parse($request))
   {
    $this->e->add("Modules request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if($api->commands[0]["modules"]!=Array())
   {
    $this->e->add("Modules parse should set modules");
    return UT_FAILED;
   }


   $api->clear();
   $request='modules
categories
users';

   if(!$api->parse($request))
   {
    $this->e->add("Modules request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if($api->commands[0]["modules"]!=Array("categories", "users"))
   {
    $this->e->add("Modules parse should set modules");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Projects_Request()
  {
   $api=new API_CSV("server", $this->_config(true));

   $request='"projects"';

   if(!$api->parse($request))
   {
    $this->e->add("Projects request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Projects parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="projects")
   {
    $this->e->add("Projects parse should set projects command");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Format_Data_Response()
  {
   global $_session;

   $csv=new Format_CSV();
   $api=new API_CSV("server", $this->_config(true));

   $d=Data::cache("categories", "view", "all");
   $d->addField("base64", "file", "show", "", 0, 10000);

   $d->setValue("nr", 1);

   $d->setShowFields("nr", "name", "text", "base64Data");
   $d->addMutilation("name", "strip", 100);
   $d->sortOrder("nr-");

   if(!$api->addData($d, true))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $api->commands[0]["data"]->setValue("name", "JAS");
   $api->commands[0]["data"]->setValue("text", "multi
line & value");
   $api->commands[0]["data"]->setValue("base64Data", "fpgmp9)");

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)!=6)
   {
    $this->e->add("Response should contain 6 rows");
    return UT_FAILED;
   }

   if(sizeof($csv->data[0])!=7)
   {
    $this->e->add("Response should contain 7 columns");
    return UT_FAILED;
   }

   if($csv->data[0][0]!="data")
   {
    $this->e->add("Data response marker doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][1]!="categories")
   {
    $this->e->add("Data response module doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][2]!="view")
   {
    $this->e->add("Data response action doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][3]!="all")
   {
    $this->e->add("Data response extra doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][4]!="html")
   {
    $this->e->add("Data response format doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][5])
   {
    $this->e->add("Data response db doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][6]!="true")
   {
    $this->e->add("Data response count doesn't match");
    return UT_FAILED;
   }

   if(sizeof($csv->data[1])!=4)
   {
    $this->e->add("Response should contain 4 columns");
    return UT_FAILED;
   }

   if($csv->data[1][0]!="nr")
   {
    $this->e->add("Data response field name doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][1]!="name")
   {
    $this->e->add("Data response field name doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][2]!="text")
   {
    $this->e->add("Data response field name doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][3]!="base64Data")
   {
    $this->e->add("Data response field name doesn't match");
    return UT_FAILED;
   }

   if(sizeof($csv->data[2])!=4)
   {
    $this->e->add("Response should contain 4 columns");
    return UT_FAILED;
   }

   if($csv->data[2][0]!="1")
   {
    $this->e->add("Data response field value doesn't match");
    return UT_FAILED;
   }

   if($csv->data[2][1]!="JAS")
   {
    $this->e->add("Data response field value doesn't match");
    return UT_FAILED;
   }

   if($csv->data[2][2]!="multi
line & value")
   {
    $this->e->add("Data response field value doesn't match");
    return UT_FAILED;
   }

   if($csv->data[2][3]!="ZnBnbcZwOSk=")
   {
    $this->e->add("Data response field value doesn't match");
    return UT_FAILED;
   }

   if($csv->data[3][0]!="")
   {
    $this->e->add("Data response empty line doesn't match");
    return UT_FAILED;
   }

   if(sizeof($csv->data[4])!=2)
   {
    $this->e->add("Response should contain 2 columns");
    return UT_FAILED;
   }

   if($csv->data[4][0]!="sort")
   {
    $this->e->add("Data response sort key doesn't match");
    return UT_FAILED;
   }

   if($csv->data[4][1]!="nr-")
   {
    $this->e->add("Data response sort field doesn't match");
    return UT_FAILED;
   }

   if(sizeof($csv->data[5])!=4)
   {
    $this->e->add("Response should contain 4 columns");
    return UT_FAILED;
   }

   if($csv->data[5][0]!="mutilation")
   {
    $this->e->add("Data response mutilation key doesn't match");
    return UT_FAILED;
   }

   if($csv->data[5][1]!="name")
   {
    $this->e->add("Data response mutilation field doesn't match");
    return UT_FAILED;
   }

   if($csv->data[5][2]!="strip")
   {
    $this->e->add("Data response mutilation type doesn't match");
    return UT_FAILED;
   }

   if($csv->data[5][3]!="100")
   {
    $this->e->add("Data response mutilation argument 1 doesn't match");
    return UT_FAILED;
   }


   $csv->clear();
   $api->clear();
   $_session->set("sessionStorage.data.unit_testing_testmodule"); //ensure no values in DB

   $d=Data::cache("unit_testing_testmodule", "add", "item", Array("db" => "session"));
   $d->setValue("name", "test");
   $d->setValue("catnr", 2);
   $d->setShowFields(); //only get nr

   if(!$api->addData($d, false))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)!=3)
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should contain 3 rows");
    return UT_FAILED;
   }

   if(sizeof($csv->data[0])!=7)
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should contain 5 columns");
    return UT_FAILED;
   }

   if(sizeof($csv->data[1])!=1)
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should contain 1 columns");
    return UT_FAILED;
   }

   if($csv->data[1][0]!="nr")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response field name doesn't match");
    return UT_FAILED;
   }

   if(sizeof($csv->data[2])!=1)
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should contain 1 columns");
    return UT_FAILED;
   }

   if($csv->data[2][0]!="1")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response field value doesn't match");
    return UT_FAILED;
   }

   $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
   return UT_OK;
  }


  function Format_Language_Response()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("server", $this->_config(true));

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)<3)
   {
    $this->e->add("Response should at least contain 3 rows");
    return UT_FAILED;
   }

   if(sizeof($csv->data[0])!=3)
   {
    $this->e->add("Response header should contain 3 columns");
    return UT_FAILED;
   }

   if(sizeof($csv->data[1])!=2)
   {
    $this->e->add("Response should contain 2 columns");
    return UT_FAILED;
   }

   if($csv->data[0][0]!="language")
   {
    $this->e->add("Language response marker doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][1]!="modules")
   {
    $this->e->add("Language response set doesn't match");
    return UT_FAILED;
   }

   if($csv->data[0][2]!="nl")
   {
    $this->e->add("Language response language doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][0]!="en")
   {
    $this->e->add("Language response original doesn't match");
    return UT_FAILED;
   }

   if($csv->data[1][1]!="nl")
   {
    $this->e->add("Language response translated doesn't match");
    return UT_FAILED;
   }

   if(!$csv->data[2][0])
   {
    $this->e->add("Language response translation original not set");
    return UT_FAILED;
   }

   if(!$csv->data[2][1])
   {
    $this->e->add("Language response translation translated not set");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Format_Modules_Response()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("server", $this->_config(true));

   if(!$api->addModules(Array()))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)<10)
   {
    $this->e->add("Response should at least contain 10 rows");
    return UT_FAILED;
   }

   if(sizeof($csv->data[0])!=1)
   {
    $this->e->add("Response header should contain 1 column");
    return UT_FAILED;
   }

   if($csv->data[0][0]!="modules")
   {
    $this->e->add("Modules response marker doesn't match");
    return UT_FAILED;
   }

   foreach(Array("categories", "users", "files") as $module)
   {
    $found=false;
    $csv->reset();
    while($csv->next())
    {
     if($csv->v[0]==$module)
     {
      $found=true;
      break;
     }
    }

    if(!$found)
    {
     $this->e->add("Modules response doesn't contain module: ".$module);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function Format_Projects_Response()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("server", $this->_config(true));

   if(!$api->addProjects(false))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->data)<2)
   {
    $this->e->add("Response should at least contain 2 rows");
    return UT_FAILED;
   }

   if(sizeof($csv->data[0])!=2)
   {
    $this->e->add("Response header should contain 2 columns");
    return UT_FAILED;
   }

   if($csv->data[0][0]!="projects")
   {
    $this->e->add("Projects response marker doesn't match");
    return UT_FAILED;
   }


   $project="jas";
   $found=false;
   $csv->reset();
   while($csv->next())
   {
    if($csv->v[0]==$project)
    {
     $found=true;
     break;
    }
   }

   if(!$found)
   {
    $this->e->add("Projects response doesn't contain project: ".$project);
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_API_CSV_Client extends UT_API_CSV
 {
  function Constructor()
  {
   $object=new API_CSV("client");

   if($object->mode!="client")
   {
    $this->e->add("API mode isn't client");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Error()
  {
   $api=new API_CSV("client", $this->_config(true));

   $e=new Errors();
   if(!$e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   $response='"errors";"Level";"Extra"
"Error";"error";"Text"';

   if(!$api->parse($response))
   {
    $this->e->add("Error response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if($api->e->getErrors()!=1)
   {
    $this->e->add("Error parse should set 1 error");
    return UT_FAILED;
   }

   if(!$this->_compare($e, $api->e))
    return UT_FAILED;

   return UT_OK;
  }


  function Version()
  {
   $api=new API_CSV("client", $this->_config(true));

   //valid XML version
   $api->e->clear();
   $response='fail';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($api->e->getLevel(0)!="warning")
   {
    $this->e->add("First error should contain: should be a warning");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version not set, using version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version not set, using version: \"1.0\"");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(1), "unsupported parse fail response")===false)
   {
    $this->e->add("Second error should contain: unsupported parse fail response");
    return UT_FAILED;
   }


   //valid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-1.0",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='fail';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail response")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail response");
    return UT_FAILED;
   }


   //invalid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='fail';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version: \"0.1a\" doesn't match version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version: \"0.1a\" doesn't match version: \"1.0\"");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Format_Data_Request()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("client", $this->_config(true));

   $d=Data::cache("categories", "view", "all", Array("format" => "api"));
   $d->addField("base64", "file", "show", "", 0, 10000);

   $d->setValue("nr", 1);
   $d->setValue("name", "JAS");
   $d->setValue("text", "multi
line & value");
   $d->setValue("base64Data", "fpgmp9)");

   $d->setShowFields("nr", "name");
   $d->addMutilation("name", "strip", 100);
   $d->sortOrder("nr-");

   if(!$api->addData($d, true))
   {
    $this->e->add("API add csv failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("data", "categories", "view", "all", "api", "", "true"),
    Array("nr", "name", "text", "base64Data"),
    Array("1", "JAS", "multi
line & value", "ZnBnbcZwOSk="),
    Array(""),
    Array("sort", "nr-"),
    Array("mutilation", "name", "strip", "100"),
   ), $csv->data))
    return UT_FAILED;

   return UT_OK;
  }

  function Format_Language_Request()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("client", $this->_config(true));

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("language", "modules", "nl"),
   ), $csv->data))
    return UT_FAILED;

   return UT_OK;
  }

  function Format_Modules_Request()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("client", $this->_config(true));

   if(!$api->addModules())
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("modules"),
   ), $csv->data))
    return UT_FAILED;


   $csv->clear();
   $api->clear();

   if(!$api->addModules(Array("categories", "users")))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("modules"),
    Array("categories"),
    Array("users")
   ), $csv->data))
    return UT_FAILED;


   $csv->clear();
   $api->clear();
   $d=Data::cache("categories", "view", "all");

   if(!$api->addModules(Array(&$d)))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("modules"),
    Array("categories"),
   ), $csv->data))
    return UT_FAILED;


   return UT_OK;
  }

  function Format_Projects_Request()
  {
   $csv=new Format_CSV();
   $api=new API_CSV("client", $this->_config(true));

   if(!$api->addProjects())
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("projects"),
   ), $csv->data))
    return UT_FAILED;


   $csv->clear();
   $api->clear();

   if(!$api->addProjects("jas"))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$csv->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(
    Array("projects", "jas"),
   ), $csv->data))
    return UT_FAILED;


   return UT_OK;
  }


  function Parse_Data_Response()
  {
   $api=new API_CSV("client", $this->_config(true));

   $response='"data";"files";"view";"all";"api"
"file";"fileData";"nr";"updated"
"JAS";"ZnBnbcZwOSk=";"1";"2013-11-18 20:49:27"
""
"sort";"updated-";"nr-"
"mutilation";"file";"strip";"100"';

   if(!$api->parse($response))
   {
    $this->e->add("Data response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Data parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="data")
   {
    $this->e->add("Data parse should set data command");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->module!="files")
   {
    $this->e->add("Response should set files module");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->action!="view")
   {
    $this->e->add("Response should set view action");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->extra!="all")
   {
    $this->e->add("Response should set view extra");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->dataformat!="api")
   {
    $this->e->add("Response should set api format");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("file")!="JAS")
   {
    $this->e->add("Response set value incorrect for field: file");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("fileData")!="fpgmp9)")
   {
    $this->e->add("Response set value incorrect for field: fileData");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("nr")!="1")
   {
    $this->e->add("Response set value incorrect for field: nr");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("updated")!="2013-11-18 20:49:27")
   {
    $this->e->add("Response set value incorrect for field: updated");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->sortOrder!=Array("updated-", "nr-"))
   {
    $this->e->add("Response set sort order fields incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->field["file"]["mutilation"]!=Array("strip" => Array(100)))
   {
    $this->e->add("Response set field mutilations incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->check("use", "show")!=Array("nr", "updated", "file", "fileData"))
   {
    $this->e->add("Response set show fields incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Language_Response()
  {
   $api=new API_CSV("client", $this->_config(true));

   $response='"language";"module";"nl"
"en";"nl"
"Test";"Test translated"
"Test |translate|";"|1| Test"
"|notranslate| test";"Test |1|"';

   if(!$api->parse($response))
   {
    $this->e->add("Language response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Language parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="language")
   {
    $this->e->add("Language parse should set language command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["raw"]["en"]["nl"]["module"])<3)
   {
    $this->e->add("Response should at least contain 3 translations");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["normal"]["en"]["nl"]["module"])<1)
   {
    $this->e->add("Response should at least contain 1 normal translations");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["lowercase"]["en"]["nl"]["module"])<1)
   {
    $this->e->add("Response should at least contain 1 lowercase translations");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["regexp"]["en"]["nl"]["module"])<2)
   {
    $this->e->add("Response should at least contain 2 regexps");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Modules_Response()
  {
   $api=new API_CSV("client", $this->_config(true));

   $response="'modules'
'categories'
'users'";

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=2)
   {
    $this->e->add("Response should contain 2 modules");
    return UT_FAILED;
   }


   $api->clear();

   $response='"modules";"users";"Users"
"field";"type";"use";"properties";"min";"max";"defaultvalue";"title";"text"
"nr";"nr";"sql;form";"owner;identifier;hidden;global;default";"0";"";"usernr";"nr";""
"username";"title";"form;sql;show;login";"username;title";"4";"15";"";"Username";""
"password";"password";"sql;form;login";"password";"7";"";"";"Password";""
"verifypassword";"password";"form";"password";"7";"";"";"Verify password";""
"backend";"boolean";"form;sql;show";"";"";"";"";"Backend login";"If this user is allowed to log in to the JAS backend"
"catnr";"nr";"sql;form";"rightsReference;global;default";"1";"";"1";"catnr";""
"usersAdditionalGroups";"nr";"form;show";"";"0";"";"usernr";"Additional user groups";"Select additional rights groups for this user"
""
"reference";"module";"to";"type";"from"
"catnr";"categories";"nr";"*:1";"catnr"
"usersAdditionalGroups";"categories";"nr";"*:*";"nr"
""';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }


   //module in command list
   $api->clear();
   if(!$api->addModules(Array("users")))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $response='"modules";"users";"Users"
"field";"type";"use";"properties";"min";"max";"defaultvalue";"title";"text"
"nr";"nr";"sql;form";"owner;identifier;hidden;global;default";"0";"";"usernr";"nr";""
"username";"title";"form;sql;show;login";"username;title";"4";"15";"";"Username";""
"password";"password";"sql;form;login";"password";"7";"";"";"Password";""
"verifypassword";"password";"form";"password";"7";"";"";"Verify password";""
"backend";"boolean";"form;sql;show";"";"";"";"";"Backend login";"If this user is allowed to log in to the JAS backend"
"catnr";"nr";"sql;form";"rightsReference;global;default";"1";"";"1";"catnr";""
"usersAdditionalGroups";"nr";"form;show";"";"0";"";"usernr";"Additional user groups";"Select additional rights groups for this user"
""
"reference";"module";"to";"type";"from"
"catnr";"categories";"nr";"*:1";"catnr"
"usersAdditionalGroups";"categories";"nr";"*:*";"nr"
""';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }

   //module definition in command list
   $api->clear();
   if(!$api->addModules(Array(Data::cache("users", false, false, Array("skipCheckTables" => true, "skipPostInit" => true)))))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $response='"modules";"users";"Users"
"field";"type";"use";"properties";"min";"max";"defaultvalue";"title";"text"
"nr";"nr";"sql;form";"identifier;hidden;global;default;owner";"1";"";"";"#";""
"added";"datetime";"sql;show";"noUpdate;default";"";"";"now";"Added";""
"updated";"datetime";"sql;show";"forceUpdate;default";"";"";"now";"Last update";""
"username";"title";"form;sql;show;login";"username;title";"4";"15";"";"Username";""
"password";"password";"sql;form;login";"password";"7";"";"";"Password";""
"verifypassword";"password";"form";"password";"7";"";"";"Verify password";""
"backend";"boolean";"form;sql;show";"";"";"";"";"Backend login";"If this user is allowed to log in to the JAS backend"
"catnr";"nr";"sql;form";"rightsReference;global;default";"1";"";"1";"Category";""
"usersAdditionalGroups";"nr";"form;show";"";"0";"";"";"Additional user groups";"Select additional rights groups for this user"
""
"reference";"module";"to";"type";"from"
"catnr";"categories";"nr";"*:1";"catnr"
"usersAdditionalGroups";"categories";"nr";"*:*";"nr"
""';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Projects_Response()
  {
   $api=new API_CSV("client", $this->_config(true));

   $response='"projects";"current"
"jas";"true"
"test";""';

   if(!$api->parse($response))
   {
    $this->e->add("Projects response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Projects parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="projects")
   {
    $this->e->add("Projects parse should set projects command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["projects"])!=2)
   {
    $this->e->add("Response should contain 2 projects");
    return UT_FAILED;
   }

   if($api->commands[0]["activate"]!="jas")
   {
    $this->e->add("Response should set active project");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
