<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/cli.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."cli.php");
 require_once($_path["include"]."fs.php");

 class UT_CLI extends UT_Classes
 {
  function _init()
  {
   global $_path;

   $this->sshTestDir=$_path["project"]["plugins"]."unit_testing/misc/.ssh/";
  }


  function Constructor()
  {
   $object=new CLI();

   if(!is_object($object)||get_class($object)!="CLI")
   {
    $this->e->add("Constructor doesn't return a CLI object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_array($object->output))
   {
    $this->e->add("Output array isn't an array", "error");
    return UT_FAILED;
   }
   if($object->output)
   {
    $this->e->add("Output array isn't empty", "error");
    return UT_FAILED;
   }

   if($object->return!==false)
   {
    $this->e->add("Return value isn't false", "error");
    return UT_FAILED;
   }

   if(!is_array($object->ssh))
   {
    $this->e->add("SSH array isn't an array", "error");
    return UT_FAILED;
   }
   if(!$object->ssh)
   {
    $this->e->add("SSH array is empty", "error");
    return UT_FAILED;
   }

   if(!$object->ssh["ssh"])
   {
    $this->e->add("SSH ssh command not set", "error");
    return UT_FAILED;
   }
   if(!$object->ssh["scp"])
   {
    $this->e->add("SSH scp command not set", "error");
    return UT_FAILED;
   }
   if(!$object->ssh["keygen"])
   {
    $this->e->add("SSH keygen command not set", "error");
    return UT_FAILED;
   }

   if($object->ssh["strict_host_key_checking"]!==false)
   {
    $this->e->add("SSH strict_host_key_checking isn't false", "error");
    return UT_FAILED;
   }

   if($object->ssh["host"]!==false)
   {
    $this->e->add("SSH host isn't false", "error");
    return UT_FAILED;
   }
   if($object->ssh["port"]!==false)
   {
    $this->e->add("SSH port isn't false", "error");
    return UT_FAILED;
   }
   if($object->ssh["user"]!==false)
   {
    $this->e->add("SSH user isn't false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new CLI();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("CLI object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->ssh=Array();
   if($orig->ssh==$copy->ssh)
   {
    $this->e->add("CLI object copy also modified original \"ssh\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("CLI object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $cli=new CLI();

   $cli->return=345;
   $cli->output="test";

   $cli->clear();

   if(!is_array($cli->output))
   {
    $this->e->add("Output array isn't an array", "error");
    return UT_FAILED;
   }
   if($cli->output)
   {
    $this->e->add("Output array isn't empty", "error");
    return UT_FAILED;
   }

   if($cli->return!==false)
   {
    $this->e->add("Return value isn't false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Exec()
  {
   $cli=new CLI();

   $return=false;
   $output=Array();
   if(!($lastline=exec("ls", $output, $return)))
   {
    $this->e->add("Exec ls command failed", "warning");
    return UT_SKIPPED;
   }

   if(!$cli->exec("ls"))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if($cli->return!=$return)
   {
    $this->e->add("Return value doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($cli->output!=$output)
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($cli->lastline!=$lastline)
   {
    $this->e->add("Last line doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Iexec()
  {
   global $_path;
   $iexecTest=$_path["project"]["plugins"]."unit_testing/misc/cli_iexec.tst";

   $cli=new CLI();

   $return=false;
   $output=Array();
   if(!($lastline=exec($iexecTest." 2>&1", $output, $return)))
   {
    $this->e->add("Exec echo command failed", "warning");
    return UT_SKIPPED;
   }

   $output[5]="in arch";

   $stdout="output
1
2
read
in arch
3
";
   $stderr="error
";

   while($cli->iexec($iexecTest)===true)
   {
    if($cli->lastline=="read")
     $cli->input("arch\n");
   }


   if($cli->return!=$return)
   {
    $this->e->add("Return value doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($cli->output!=$output)
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($cli->lastline!=$lastline)
   {
    $this->e->add("Last line doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($cli->stdout!=$stdout)
   {
    $this->e->add("Stderr value doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($cli->stderr!=$stderr)
   {
    $this->e->add("Stderr value doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Home()
  {
   $cli=new CLI();

   $info=posix_getpwuid(posix_getuid());

   if(!($home=$cli->home()))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if($home!=$info["dir"]."/")
   {
    $this->e->add("Home doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Whoami()
  {
   $cli=new CLI();

   $info=posix_getpwuid(posix_getuid());

   if(!($home=$cli->whoami()))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if($home!=$info["name"])
   {
    $this->e->add("Whoami doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SshFiles()
  {
   $this->_init();

   $cli=new CLI();
   if(!$cli->sshFiles($this->sshTestDir))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!file_exists($this->sshTestDir))
   {
    $this->e->add("SSH path doesn't exist", "error");
    return UT_FAILED;
   }

   //cleanup
   FS::deleteS($this->sshTestDir, true);

   if($cli->ssh["path"]!=$this->sshTestDir)
   {
    $this->e->add("SSH path doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["private_key"]!=$this->sshTestDir."id_rsa")
   {
    $this->e->add("SSH private_key doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["public_key"]!=$this->sshTestDir."id_rsa.pub")
   {
    $this->e->add("SSH public_key doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["known_hosts"]!=$this->sshTestDir."known_hosts")
   {
    $this->e->add("SSH known_hosts doesn't match expected output", "error");
    return UT_FAILED;
   }

   //try dsa keys
   $cli->ssh["keytype"]="dsa";

   if(!$cli->sshFiles($this->sshTestDir))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!file_exists($this->sshTestDir))
   {
    $this->e->add("SSH path doesn't exist", "error");
    return UT_FAILED;
   }

   //cleanup
   FS::deleteS($this->sshTestDir, true);

   if($cli->ssh["path"]!=$this->sshTestDir)
   {
    $this->e->add("SSH path doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["private_key"]!=$this->sshTestDir."id_dsa")
   {
    $this->e->add("SSH private_key doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["public_key"]!=$this->sshTestDir."id_dsa.pub")
   {
    $this->e->add("SSH public_key doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["known_hosts"]!=$this->sshTestDir."known_hosts")
   {
    $this->e->add("SSH known_hosts doesn't match expected output", "error");
    return UT_FAILED;
   }


   //custom files
   if(!$cli->sshFiles($this->sshTestDir, $this->sshTestDir."priv", $this->sshTestDir."pub", $this->sshTestDir."knwn"))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!file_exists($this->sshTestDir))
   {
    $this->e->add("SSH path doesn't exist", "error");
    return UT_FAILED;
   }

   //cleanup
   FS::deleteS($this->sshTestDir, true);

   if($cli->ssh["path"]!=$this->sshTestDir)
   {
    $this->e->add("SSH path doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["private_key"]!=$this->sshTestDir."priv")
   {
    $this->e->add("SSH private_key doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["public_key"]!=$this->sshTestDir."pub")
   {
    $this->e->add("SSH public_key doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["known_hosts"]!=$this->sshTestDir."knwn")
   {
    $this->e->add("SSH known_hosts doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SshSetup()
  {
   $this->_init();

   $cli=new CLI();
   if(!$cli->sshFiles($this->sshTestDir))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!$cli->sshSetup())
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["path"]))
   {
    $this->e->add("SSH path doesn't exist", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(FS::getModeS($cli->ssh["path"])!="700")
   {
    $this->e->add("SSH path permissions incorrect", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["private_key"]))
   {
    $this->e->add("SSH private_key doesn't exist", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(FS::getModeS($cli->ssh["private_key"])!="400")
   {
    $this->e->add("SSH private_key permissions incorrect", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(!preg_match("/^-----BEGIN RSA PRIVATE KEY-----/", FS::readS($cli->ssh["private_key"])))
   {
    $this->e->add("SSH private_key isn't a RSA key", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["public_key"]))
   {
    $this->e->add("SSH public_key doesn't exist", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(FS::getModeS($cli->ssh["public_key"])!="400")
   {
    $this->e->add("SSH public_key permissions incorrect", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(!preg_match("/^ssh-rsa/", FS::readS($cli->ssh["public_key"])))
   {
    $this->e->add("SSH public_key isn't a RSA key", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }


   FS::deleteS($this->sshTestDir, true);

   //try dsa keys
   $cli->ssh["keytype"]="dsa";

   if(!$cli->sshFiles($this->sshTestDir))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!$cli->sshSetup())
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["path"]))
   {
    $this->e->add("SSH path doesn't exist", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(FS::getModeS($cli->ssh["path"])!="700")
   {
    $this->e->add("SSH path permissions incorrect", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["private_key"]))
   {
    $this->e->add("SSH private_key doesn't exist", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(FS::getModeS($cli->ssh["private_key"])!="400")
   {
    $this->e->add("SSH private_key permissions incorrect", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(!preg_match("/^-----BEGIN DSA PRIVATE KEY-----/", FS::readS($cli->ssh["private_key"])))
   {
    $this->e->add("SSH private_key isn't a DSA key", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["public_key"]))
   {
    $this->e->add("SSH public_key doesn't exist", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(FS::getModeS($cli->ssh["public_key"])!="400")
   {
    $this->e->add("SSH public_key permissions incorrect", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }
   if(!preg_match("/^ssh-dss/", FS::readS($cli->ssh["public_key"])))
   {
    $this->e->add("SSH public_key isn't a DSA key", "error");
    FS::deleteS($this->sshTestDir, true);
    return UT_FAILED;
   }

   FS::deleteS($this->sshTestDir, true);
   return UT_OK;
  }


  function Ssh()
  {
   $this->_init();

   $cli=new CLI();

   if(!$cli->ssh("localhost"))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if(!file_exists($cli->ssh["private_key"]))
   {
    $this->e->add("SSH private_key doesn't exist", "error");
    return UT_FAILED;
   }
   if(!file_exists($cli->ssh["public_key"]))
   {
    $this->e->add("SSH public_key doesn't exist", "error");
    return UT_FAILED;
   }

   if($cli->ssh["host"]!="localhost")
   {
    $this->e->add("Host value doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["user"])
   {
    $this->e->add("User value doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["port"])
   {
    $this->e->add("Port value doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["strict_host_key_checking"])
   {
    $this->e->add("Strict_host_key_checking value doesn't match expected output", "error");
    return UT_FAILED;
   }


   if(!$cli->ssh("localhost", "nobody", 443, true))
   {
    $this->e->add($cli->e);
    return UT_FAILED;
   }

   if($cli->ssh["host"]!="localhost")
   {
    $this->e->add("Host value doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["user"]!="nobody")
   {
    $this->e->add("User value doesn't match expected output", "error");
    return UT_FAILED;
   }
   if($cli->ssh["port"]!="443")
   {
    $this->e->add("Port value doesn't match expected output", "error");
    return UT_FAILED;
   }
   if(!$cli->ssh["strict_host_key_checking"])
   {
    $this->e->add("Strict_host_key_checking value doesn't match expected output", "error");
    return UT_FAILED;
   }


   if($cli->ssh("local/q124"))
   {
    $this->e->add("SSH should fail on illegal hostname");
    return UT_FAILED;
   }

   if($cli->ssh("localhost", "1m52;2efaf`213"))
   {
    $this->e->add("SSH should fail on illegal username");
    return UT_FAILED;
   }

   if($cli->ssh("localhost", false, "abdc"))
   {
    $this->e->add("SSH should fail on illegal port number");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SshExec()
  {
   $cli=new CLI();
   $homeDir=$cli->home();
   $sshDir=$homeDir.".ssh/";

   if(is_dir($sshDir))
   {
    if(FS::filesS($sshDir))
    {
     $this->e->add("Test ssh directory: \"".$sshDir."\" already in use", "warning");
     return UT_SKIPPED;
    }
   }
   else if(!FS::mkdirS($sshDir))
   {
    $this->e->add("Unable to create ssh directory: \"".$sshDir."\"", "warning");
    return UT_SKIPPED;
   }

   if(!$cli->ssh("localhost", $cli->whoami()))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!FS::copyS($cli->ssh["public_key"], $sshDir."authorized_keys"))
   {
    $this->e->add("Can't copy public key to ssh directory: \"".$sshDir."\"", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!$cli->exec("ls ~/"))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   $remoteFiles=$cli->output;

   $files=FS::filesS($homeDir."/", false, true);
   sort($files);

   if($remoteFiles!=$files)
   {
    $this->e->add("SSH command doesn't match expected output", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   FS::deleteS($sshDir, true);

   if(!file_exists($cli->ssh["known_hosts"]))
   {
    $this->e->add("SSH known_hosts doesn't exist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SshIexec()
  {
   $cli=new CLI();
   $homeDir=$cli->home();
   $sshDir=$homeDir.".ssh/";

   if(is_dir($sshDir))
   {
    if(FS::filesS($sshDir))
    {
     $this->e->add("Test ssh directory: \"".$sshDir."\" already in use", "warning");
     return UT_SKIPPED;
    }
   }
   else if(!FS::mkdirS($sshDir))
   {
    $this->e->add("Unable to create ssh directory: \"".$sshDir."\"", "warning");
    return UT_SKIPPED;
   }

   if(!$cli->ssh("localhost", $cli->whoami()))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!FS::copyS($cli->ssh["public_key"], $sshDir."authorized_keys"))
   {
    $this->e->add("Can't copy public key to ssh directory: \"".$sshDir."\"", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   while($cli->iexec("ls ~/")===true)
   {
    //...
   }

   if($cli->return===false)
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   $remoteFiles=$cli->output;

   $files=FS::filesS($homeDir."/", false, true);
   sort($files);

   if($remoteFiles!=$files)
   {
    $this->e->add("SSH command doesn't match expected output", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   FS::deleteS($sshDir, true);

   if(!file_exists($cli->ssh["known_hosts"]))
   {
    $this->e->add("SSH known_hosts doesn't exist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Scp()
  {
   $cli=new CLI();
   $homeDir=$cli->home();
   $sshDir=$homeDir.".ssh/";

   if(is_dir($sshDir))
   {
    if(FS::filesS($sshDir))
    {
     $this->e->add("Test ssh directory: \"".$sshDir."\" already in use", "warning");
     return UT_SKIPPED;
    }
   }
   else if(!FS::mkdirS($sshDir))
   {
    $this->e->add("Unable to create ssh directory: \"".$sshDir."\"", "warning");
    return UT_SKIPPED;
   }

   if(!$cli->ssh("localhost", $cli->whoami()))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!FS::copyS($cli->ssh["public_key"], $sshDir."authorized_keys"))
   {
    $this->e->add("Can't copy public key to ssh directory: \"".$sshDir."\"", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }


   if(!$cli->scp("index.php", false, true)) //copy index.php to remote home
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!$cli->exec("ls ~/"))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."index.php");
    return UT_FAILED;
   }

   if(!in_array("index.php", $cli->output))
   {
    $this->e->add("SCP command doesn't match expected output", "error");
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."index.php");
    return UT_FAILED;
   }


   if(!$cli->scp("index.php", "___lala.php", false)) //copy remote index.php to local ___lala.php
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."index.php");
    return UT_FAILED;
   }

   if(!in_array("___lala.php", FS::filesS("./", false, true)))
   {
    $this->e->add("SCP command doesn't match expected output", "error");
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."index.php");
    FS::deleteS("___lala.php");
    return UT_FAILED;
   }

   FS::deleteS($homeDir."index.php");
   FS::deleteS("___lala.php");

   if($cli->scp("___lala.php", false, true)) //copy non existing file to remote home
   {
    $this->e->add("SCP shouldn't copy non-existing file", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   FS::deleteS($sshDir, true);

   if(!file_exists($cli->ssh["known_hosts"]))
   {
    $this->e->add("SSH known_hosts doesn't exist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ScpLR()
  {
   $cli=new CLI();
   $homeDir=$cli->home();
   $sshDir=$homeDir.".ssh/";

   if(is_dir($sshDir))
   {
    if(FS::filesS($sshDir))
    {
     $this->e->add("Test ssh directory: \"".$sshDir."\" already in use", "warning");
     return UT_SKIPPED;
    }
   }
   else if(!FS::mkdirS($sshDir))
   {
    $this->e->add("Unable to create ssh directory: \"".$sshDir."\"", "warning");
    return UT_SKIPPED;
   }

   if(!$cli->ssh("localhost", $cli->whoami()))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!FS::copyS($cli->ssh["public_key"], $sshDir."authorized_keys"))
   {
    $this->e->add("Can't copy public key to ssh directory: \"".$sshDir."\"", "error");
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }


   if(!$cli->scpl("index.php", "___lala.php")) //copy index.php to remote home ___lala.php
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    return UT_FAILED;
   }

   if(!$cli->exec("ls ~/"))
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."___lala.php");
    return UT_FAILED;
   }

   if(!in_array("___lala.php", $cli->output))
   {
    $this->e->add("SCP command doesn't match expected output", "error");
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."___lala.php");
    return UT_FAILED;
   }


   if(!$cli->scpr("___lala.php", false)) //copy remote ___lala.php to local ___lala.php
   {
    $this->e->add($cli->e);
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."___lala.php");
    return UT_FAILED;
   }

   if(!in_array("___lala.php", FS::filesS("./", false, true)))
   {
    $this->e->add("SCP command doesn't match expected output", "error");
    FS::deleteS($sshDir, true);
    FS::deleteS($homeDir."___lala.php");
    FS::deleteS("___lala.php");
    return UT_FAILED;
   }

   FS::deleteS($homeDir."___lala.php");
   FS::deleteS("___lala.php");
   FS::deleteS($sshDir, true);

   if(!file_exists($cli->ssh["known_hosts"]))
   {
    $this->e->add("SSH known_hosts doesn't exist", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
