<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/form.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."form.php");

 class UT_Form extends UT_Classes
 {
  function Constructor()
  {
   $object=new Form();

   if(!is_object($object)||get_class($object)!="Form")
   {
    $this->e->add("Constructor doesn't return a Form object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Form();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Form object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->form=-1;
   if($orig->form==$copy->form)
   {
    $this->e->add("Form object copy also modified original \"form\" variable", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("Form object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Start()
  {
   global $_path;

   $f=new Form();

   if(!($output=$f->start()))
   {
    $this->e->add("Form start with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<form action=\"".$_path["HTML"]["url"]."\" method=\"post\" enctype=\"multipart/form-data\"><div class=\"invisible\">", $output))
   {
    $this->e->add("Form start with no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->start("url", "get", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form start with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<form action=\"url\" method=\"get\" enctype=\"multipart/form-data\" class=\"class\" id=\"id\" style=\"width: 100%\"><div class=\"invisible\">", $output))
   {
    $this->e->add("Form start with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Input()
  {
   $f=new Form();

//hidden
   if(!($output=$f->input()))
   {
    $this->e->add("Form input hidden with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"hidden\" name=\"\" value=\"\"/>", $output))
   {
    $this->e->add("Form input hidden no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("hidden", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input hidden with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"hidden\" name=\"name\" value=\"value\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input hidden with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("hidden", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input hidden checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input hidden checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//button
   if(!($output=$f->input("button")))
   {
    $this->e->add("Form input button with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"button\" name=\"\" value=\"\" class=\"form button\"/>", $output))
   {
    $this->e->add("Form input button no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("button", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input button with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"button\" name=\"name\" value=\"value\" class=\"class button\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input button with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("button", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input button checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"button\" name=\"name\" value=\"value\" class=\"form button\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input button checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("button", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input button disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"button\" name=\"name\" value=\"value\" class=\"form button\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input button disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//checkbox
   if(!($output=$f->input("checkbox")))
   {
    $this->e->add("Form input checkbox with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"checkbox\" name=\"\" value=\"\" class=\"form checkbox\" style=\"width: 15px; height: 15px;\"/>", $output))
   {
    $this->e->add("Form input checkbox no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("checkbox", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input checkbox with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"checkbox\" name=\"name\" value=\"value\" class=\"class checkbox\" id=\"id\" style=\"width: 15px; height: 15px; width: 100%\"/>", $output))
   {
    $this->e->add("Form input checkbox with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("checkbox", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input checkbox checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"checkbox\" name=\"name\" value=\"value\" class=\"form checkbox\" style=\"width: 15px; height: 15px;\" checked=\"checked\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input checkbox checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("checkbox", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input checkbox disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"checkbox\" name=\"name\" value=\"value\" class=\"form checkbox\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\"/>", $output))
   {
    $this->e->add("Form input checkbox checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//email
   if(!($output=$f->input("email")))
   {
    $this->e->add("Form input email with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"email\" name=\"\" value=\"\" class=\"form email\"/>", $output))
   {
    $this->e->add("Form input email no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("email", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input email with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"email\" name=\"name\" value=\"value\" class=\"class email\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input email with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("email", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input email checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"email\" name=\"name\" value=\"value\" class=\"form email\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input email checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("email", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input email disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"email\" name=\"name\" value=\"value\" class=\"form email\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input email disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//file
   if(!($output=$f->input("file")))
   {
    $this->e->add("Form input file with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"file\" name=\"\" value=\"\" class=\"form file\"/>", $output))
   {
    $this->e->add("Form input file no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("file", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input file with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"file\" name=\"name\" value=\"value\" class=\"class file\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input file with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("file", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input file checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"file\" name=\"name\" value=\"value\" class=\"form file\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input file checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("file", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input file disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"file\" name=\"name\" value=\"value\" class=\"form file\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input file disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//image
   if(!($output=$f->input("image")))
   {
    $this->e->add("Form input image with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"image\" name=\"\" value=\"\" class=\"form image\"/>", $output))
   {
    $this->e->add("Form input image no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("image", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input image with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"image\" name=\"name\" value=\"value\" class=\"class image\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input image with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("image", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input image checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"image\" name=\"name\" value=\"value\" class=\"form image\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input image checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("image", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input image disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"image\" name=\"name\" value=\"value\" class=\"form image\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input image disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//password
   if(!($output=$f->input("password")))
   {
    $this->e->add("Form input password with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"password\" name=\"\" value=\"\" class=\"form password\"/>", $output))
   {
    $this->e->add("Form input password no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("password", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input password with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"password\" name=\"name\" value=\"value\" class=\"class password\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input password with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("password", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input password checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"password\" name=\"name\" value=\"value\" class=\"form password\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input password checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("password", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input password disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"password\" name=\"name\" value=\"value\" class=\"form password\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input password disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//radio
   if(!($output=$f->input("radio")))
   {
    $this->e->add("Form input radio with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"radio\" name=\"\" value=\"\" class=\"form radio\" style=\"width: 15px; height: 15px;\"/>", $output))
   {
    $this->e->add("Form input radio no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("radio", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input radio with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"radio\" name=\"name\" value=\"value\" class=\"class radio\" id=\"id\" style=\"width: 15px; height: 15px; width: 100%\"/>", $output))
   {
    $this->e->add("Form input radio with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("radio", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input radio checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"radio\" name=\"name\" value=\"value\" class=\"form radio\" style=\"width: 15px; height: 15px;\" checked=\"checked\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input radio checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("radio", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input radio disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"radio\" name=\"name\" value=\"value\" class=\"form radio\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\"/>", $output))
   {
    $this->e->add("Form input radio disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//reset
   if(!($output=$f->input("reset")))
   {
    $this->e->add("Form input reset with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"reset\" name=\"\" value=\"\" class=\"form reset\"/>", $output))
   {
    $this->e->add("Form input reset no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("reset", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input reset with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"reset\" name=\"name\" value=\"value\" class=\"class reset\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input reset with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("reset", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input reset checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"reset\" name=\"name\" value=\"value\" class=\"form reset\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input reset checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("reset", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input reset disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"reset\" name=\"name\" value=\"value\" class=\"form reset\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input reset disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//search
   if(!($output=$f->input("search")))
   {
    $this->e->add("Form input search with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"search\" name=\"\" value=\"\" class=\"form search\"/>", $output))
   {
    $this->e->add("Form input search no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("search", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input search with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"search\" name=\"name\" value=\"value\" class=\"class search\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input search with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("search", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input search checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"search\" name=\"name\" value=\"value\" class=\"form search\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input search checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("search", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input search disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"search\" name=\"name\" value=\"value\" class=\"form search\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input search disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//submit
   if(!($output=$f->input("submit")))
   {
    $this->e->add("Form input submit with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"submit\" name=\"\" value=\"\" class=\"form submit\"/>", $output))
   {
    $this->e->add("Form input submit no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("submit", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input submit with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"submit\" name=\"name\" value=\"value\" class=\"class submit\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input submit with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("submit", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input submit checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"submit\" name=\"name\" value=\"value\" class=\"form submit\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input submit checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("submit", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input submit disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"submit\" name=\"name\" value=\"value\" class=\"form submit\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input submit disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//text
   if(!($output=$f->input("text")))
   {
    $this->e->add("Form input text with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"text\" name=\"\" value=\"\" class=\"form text\"/>", $output))
   {
    $this->e->add("Form input text no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("text", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input text with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"text\" name=\"name\" value=\"value\" class=\"class text\" id=\"id\" style=\"width: 100%\"/>", $output))
   {
    $this->e->add("Form input text with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("text", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input text checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"text\" name=\"name\" value=\"value\" class=\"form text\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input text checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("text", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input text disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<input type=\"text\" name=\"name\" value=\"value\" class=\"form text\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input text disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//textarea
   if(!($output=$f->input("textarea")))
   {
    $this->e->add("Form input textarea with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<textarea name=\"\" class=\"form textarea\" cols=\"\" rows=\"\"></textarea>", $output))
   {
    $this->e->add("Form input textarea no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("textarea", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input textarea with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<textarea name=\"name\" class=\"class textarea\" id=\"id\" style=\"width: 100%\" cols=\"\" rows=\"\">value</textarea>", $output))
   {
    $this->e->add("Form input textarea with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("textarea", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input textarea checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<textarea name=\"name\" class=\"form textarea\" disabled=\"disabled\" cols=\"\" rows=\"\">value</textarea><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input textarea checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("textarea", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input textarea disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<textarea name=\"name\" class=\"form textarea\" disabled=\"disabled\" cols=\"\" rows=\"\">value</textarea><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form input textarea disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

//fake
   if(!($output=$f->input("fake")))
   {
    $this->e->add("Form input fake with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<p class=\"form fake\">&nbsp; </p>", $output))
   {
    $this->e->add("Form input fake no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("fake", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form input fake with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<p class=\"class fake\" id=\"id\" style=\"width: 100%\">value</p>", $output))
   {
    $this->e->add("Form input fake with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("fake", "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form input fake checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<p class=\"form fake\">value</p>", $output))
   {
    $this->e->add("Form input fake checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->input("fake", "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form input fake disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<p class=\"form fake\">value</p>", $output))
   {
    $this->e->add("Form input fake disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Button()
  {
   $f=new Form();

   if(!($output=$f->button()))
   {
    $this->e->add("Form button with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<button class=\"form button\" type=\"button\"></button>", $output))
   {
    $this->e->add("Form button no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->button("name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form button with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<button name=\"name\" value=\"value\" class=\"class button\" id=\"id\" type=\"button\" style=\"width: 100%\">value</button>", $output))
   {
    $this->e->add("Form button with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->button("name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form button checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<button name=\"name\" value=\"value\" class=\"form button\" type=\"button\" disabled=\"disabled\">value</button>", $output))
   {
    $this->e->add("Form button checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->button("name", "<img />value", false, false, "reset disabled")))
   {
    $this->e->add("Form button reset/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<button name=\"name\" value=\"value\" class=\"form button\" type=\"reset\" disabled=\"disabled\"><img />value</button>", $output))
   {
    $this->e->add("Form button reset/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->button("name", "<img />", false, false, "submit disabled")))
   {
    $this->e->add("Form button submit shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<button name=\"name\" class=\"form button\" type=\"submit\" disabled=\"disabled\"><img /></button>", $output))
   {
    $this->e->add("Form button submit output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Checkbox()
  {
   $f=new Form();

   if(!($output=$f->checkbox()))
   {
    $this->e->add("Form checkbox with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id=preg_replace("/.*id=\"(checkbox\d+)\".*/", "\\1", $output);
   if(!$this->_compare("<input type=\"checkbox\" name=\"\" value=\"\" class=\"form checkbox\" id=\"".$id."\" style=\"width: 15px; height: 15px;\"/>", $output))
   {
    $this->e->add("Form checkbox no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->checkbox(true, "Text", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form checkbox with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<label for=\"id\"><input type=\"checkbox\" name=\"name\" value=\"value\" class=\"class checkbox\" id=\"id\" style=\"width: 15px; height: 15px; width: 100%\" checked=\"checked\"/> Text</label>", $output))
   {
    $this->e->add("Form checkbox with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->checkbox(false, false, "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form checkbox checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id=preg_replace("/.*id=\"(checkbox\d+)\".*/", "\\1", $output);
   if(!$this->_compare("<input type=\"checkbox\" name=\"name\" value=\"value\" class=\"form checkbox\" id=\"".$id."\" style=\"width: 15px; height: 15px;\" checked=\"checked\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form checkbox checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->checkbox(false, false, "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form checkbox disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id=preg_replace("/.*id=\"(checkbox\d+)\".*/", "\\1", $output);
   if(!$this->_compare("<input type=\"checkbox\" name=\"name\" value=\"value\" class=\"form checkbox\" id=\"".$id."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\"/>", $output))
   {
    $this->e->add("Form checkbox disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Radio()
  {
   $f=new Form();

   if(!($output=$f->radio()))
   {
    $this->e->add("Form radio with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id=preg_replace("/.*id=\"(radio\d+)\".*/", "\\1", $output);
   if(!$this->_compare("<input type=\"radio\" name=\"\" value=\"\" class=\"form radio\" id=\"".$id."\" style=\"width: 15px; height: 15px;\"/>", $output))
   {
    $this->e->add("Form radio no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->radio(true, "Text", "name", "value", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form radio with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<label for=\"id\"><input type=\"radio\" name=\"name\" value=\"value\" class=\"class radio\" id=\"id\" style=\"width: 15px; height: 15px; width: 100%\" checked=\"checked\"/> Text</label>", $output))
   {
    $this->e->add("Form radio with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->radio(false, false, "name", "value", false, false, "checked disabled")))
   {
    $this->e->add("Form radio checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id=preg_replace("/.*id=\"(radio\d+)\".*/", "\\1", $output);
   if(!$this->_compare("<input type=\"radio\" name=\"name\" value=\"value\" class=\"form radio\" id=\"".$id."\" style=\"width: 15px; height: 15px;\" checked=\"checked\" disabled=\"disabled\"/><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form radio checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->radio(false, false, "name", "value", false, false, "disabled")))
   {
    $this->e->add("Form radio disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id=preg_replace("/.*id=\"(radio\d+)\".*/", "\\1", $output);
   if(!$this->_compare("<input type=\"radio\" name=\"name\" value=\"value\" class=\"form radio\" id=\"".$id."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\"/>", $output))
   {
    $this->e->add("Form radio disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SelectStart()
  {
   $f=new Form();

   if(!($output=$f->selectStart()))
   {
    $this->e->add("Form select start with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"\" class=\"form select\">", $output))
   {
    $this->e->add("Form select start no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->selectStart(true, "name", "class", "id", "style=\"width: 100%\"")))
   {
    $this->e->add("Form select start with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"name[]\" class=\"class multi\" id=\"id\" style=\"width: 100%\" multiple=\"multiple\">", $output))
   {
    $this->e->add("Form select start with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->selectStart(false, "name", false, false, "checked disabled")))
   {
    $this->e->add("Form select start checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"name\" class=\"form select\" disabled=\"disabled\">", $output))
   {
    $this->e->add("Form select start checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->selectStart(false, "name", false, false, "disabled multiple")))
   {
    $this->e->add("Form select start multiple/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"name[]\" class=\"form select\" disabled=\"disabled\" multiple=\"multiple\">", $output))
   {
    $this->e->add("Form select start multiple/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SelectOption()
  {
   $f=new Form();

   if(!($output=$f->selectOption("value")))
   {
    $this->e->add("Form select option with 1 argument shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<option value=\"value\">value</option>", $output))
   {
    $this->e->add("Form select option 1 argument output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->selectOption("value", "Text", true, "style=\"width: 100%\"")))
   {
    $this->e->add("Form select option with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<option value=\"value\" style=\"width: 100%\" selected=\"selected\">Text</option>", $output))
   {
    $this->e->add("Form select option with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->selectOption("value", false, false, "checked disabled")))
   {
    $this->e->add("Form select option checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<option value=\"value\" selected=\"selected\" disabled=\"disabled\">value</option>", $output))
   {
    $this->e->add("Form select option checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->selectOption("value", false, false, "selected")))
   {
    $this->e->add("Form select option selected shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<option value=\"value\" selected=\"selected\">value</option>", $output))
   {
    $this->e->add("Form select option select output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SelectEnd()
  {
   $f=new Form();

   if(!($output=$f->selectEnd()))
   {
    $this->e->add("Form select end shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("</select>", $output))
   {
    $this->e->add("Form select end output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Select()
  {
   $f=new Form();

   if(!($output=$f->select()))
   {
    $this->e->add("Form select with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"\" class=\"form select\"></select>", $output))
   {
    $this->e->add("Form select no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->select(true, "name", Array("value", "value2"), "value", "class", "id", "style=\"width: 100%\"", true)))
   {
    $this->e->add("Form select with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"name[]\" class=\"class multi\" id=\"id\" style=\"width: 100%\" multiple=\"multiple\"><option value=\"\"></option><option value=\"value\" selected=\"selected\">value</option><option value=\"value2\">value2</option></select>", $output))
   {
    $this->e->add("Form select with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->select(false, "name", Array("value" => "Value", "value2" => "Value 2"), "value", false, false, "checked disabled")))
   {
    $this->e->add("Form select checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"name\" class=\"form select\" disabled=\"disabled\"><option value=\"value\" selected=\"selected\">Value</option><option value=\"value2\">Value 2</option></select><input type=\"hidden\" name=\"name\" value=\"value\"/>", $output))
   {
    $this->e->add("Form select checked/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->select(false, "name", Array("value", "value2", "value3"), Array("value", "value3"), false, false, "disabled multiple")))
   {
    $this->e->add("Form select multiple/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<select name=\"name[]\" class=\"form select\" disabled=\"disabled\" multiple=\"multiple\"><option value=\"value\" selected=\"selected\">value</option><option value=\"value2\">value2</option><option value=\"value3\" selected=\"selected\">value3</option></select><input type=\"hidden\" name=\"name[]\" value=\"value\"/><input type=\"hidden\" name=\"name[]\" value=\"value3\"/>", $output))
   {
    $this->e->add("Form select multiple/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ListView()
  {
   $f=new Form();

   if(!($output=$f->listview()))
   {
    $this->e->add("Form listview with no arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("<div class=\"form listview \"></div>", $output))
   {
    $this->e->add("Form listview no arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->listview(true, "name", Array("value", "value2"), "value", "class", "id", "style=\"width: 100%\"", true)))
   {
    $this->e->add("Form listview with arguments shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id1=preg_replace("/.*?id=\"(checkbox\d+)\".*/", "\\1", $output, 1);
   $id2=preg_replace("/.*?id=\"checkbox.*?id=\"(checkbox\d+)\".*/", "\\1", $output, 1);
   $id3=preg_replace("/.*?id=\"checkbox.*?id=\"checkbox.*?id=\"(checkbox\d+)\".*/", "\\1", $output, 1);
   if(!$this->_compare("<div class=\"class multilistview name\" id=\"id\" style=\"width: 100%\"><input type=\"checkbox\" name=\"name[]\" value=\"\" class=\"form checkbox\" id=\"".$id1."\" style=\"width: 15px; height: 15px; width: 100%\"/><br/><label for=\"".$id2."\"><input type=\"checkbox\" name=\"name[]\" value=\"value\" class=\"form checkbox\" id=\"".$id2."\" style=\"width: 15px; height: 15px; width: 100%\" checked=\"checked\"/> value</label><br/><label for=\"".$id3."\"><input type=\"checkbox\" name=\"name[]\" value=\"value2\" class=\"form checkbox\" id=\"".$id3."\" style=\"width: 15px; height: 15px; width: 100%\"/> value2</label><br/></div>", $output))
   {
    $this->e->add("Form listview with arguments output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->listview(false, "name", Array("value" => "Value", "value2" => "Value 2"), "value", false, false, "checked disabled")))
   {
    $this->e->add("Form listview checked/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id1=preg_replace("/.*?id=\"(radio\d+)\".*/", "\\1", $output, 1);
   $id2=preg_replace("/.*?id=\"radio.*?id=\"(radio\d+)\".*/", "\\1", $output, 1);
   if(!$this->_compare("<div class=\"form listview name\" disabled=\"disabled\"><label for=\"".$id1."\"><input type=\"radio\" name=\"name\" value=\"value\" class=\"form radio\" id=\"".$id1."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\" checked=\"checked\"/><input type=\"hidden\" name=\"name\" value=\"value\"/> Value</label><br/><label for=\"".$id2."\"><input type=\"radio\" name=\"name\" value=\"value2\" class=\"form radio\" id=\"".$id2."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\"/> Value 2</label><br/></div>", $output))
   {
    $this->e->add("Form listview checkbox/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if(!($output=$f->listview(false, "name", Array("value", "value2", "value3"), Array("value", "value3"), false, false, "disabled multiple")))
   {
    $this->e->add("Form listview multiple/disabled shouldn't have failed", "error");
    return UT_FAILED;
   }
   $id1=preg_replace("/.*?id=\"(checkbox\d+)\".*/", "\\1", $output, 1);
   $id2=preg_replace("/.*?id=\"checkbox.*?id=\"(checkbox\d+)\".*/", "\\1", $output, 1);
   $id3=preg_replace("/.*?id=\"checkbox.*?id=\"checkbox.*?id=\"(checkbox\d+)\".*/", "\\1", $output, 1);
   if(!$this->_compare("<div class=\"form multilistview name\" disabled=\"disabled\"><label for=\"".$id1."\"><input type=\"checkbox\" name=\"name[]\" value=\"value\" class=\"form checkbox\" id=\"".$id1."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\" checked=\"checked\"/><input type=\"hidden\" name=\"name[]\" value=\"value\"/> value</label><br/><label for=\"".$id2."\"><input type=\"checkbox\" name=\"name[]\" value=\"value2\" class=\"form checkbox\" id=\"".$id2."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\"/> value2</label><br/><label for=\"".$id3."\"><input type=\"checkbox\" name=\"name[]\" value=\"value3\" class=\"form checkbox\" id=\"".$id3."\" style=\"width: 15px; height: 15px;\" disabled=\"disabled\" checked=\"checked\"/><input type=\"hidden\" name=\"name[]\" value=\"value3\"/> value3</label><br/></div>", $output))
   {
    $this->e->add("Form listview multiple/disabled output doesn't match", "error", $output);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DataSelect()
  {
   return UT_SKIPPED;
  }

  function DataListView()
  {
   return UT_SKIPPED;
  }

  function OrderListValues()
  {
   return UT_SKIPPED;
  }

  function WYSIWYG()
  {
   return UT_SKIPPED;
  }

  function CheckWYSIWYG()
  {
   return UT_SKIPPED;
  }

  function Rights()
  {
   return UT_SKIPPED;
  }

  function End()
  {
   $f=new Form();

   if(!($output=$f->end()))
   {
    $this->e->add("Form end shouldn't have failed", "error");
    return UT_FAILED;
   }
   if(!$this->_compare("</div></form>", $output))
   {
    $this->e->add("Form end output doesn't match", "error", $output);
    return UT_FAILED;
   }

   if($f->form!=1)
   {
    $this->e->add("Form end counter doesn't match", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
