<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/storage.mysql.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["plugins"]."unit_testing/ut/storage.php");
 require_once($_path["include"]."storage/mysql.php");
 require_once($_path["include"]."data.php");

 class UT_Storage_MySQL extends UT_Storage
 {
  function Constructor()
  {
   $d=new Data();
   $object=new System_Storage_mysql($d, false);

   if(!is_object($object)||get_class($object)!="System_Storage_mysql")
   {
    $this->e->add("Constructor doesn't return a System_Storage_mysql object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $d=Data::cache();
   $orig=new System_Storage_mysql($d);
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("System_Storage_mysql object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->type="different";
   if($orig->type==$copy->type)
   {
    $this->e->add("System_Storage_mysql object copy also modified original \"type\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("System_Storage_mysql object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("System_Storage_mysql object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("System_Storage_mysql object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("System_Storage_mysql object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->data->format=Array();
   if($orig->data->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_mysql object copy didn't modify original \"Data\" object", "error");
    return UT_FAILED;
   }

   if($d->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_mysql object copy didn't modify referenced \"Data\" object", "error");
    return UT_FAILED;
   }

   $copy->dbcon="test";
   if($orig->dbcon==$copy->dbcon)
   {
    $this->e->add("System_Storage_mysql object copy also modified original \"MySQL link\" object", "error");
    return UT_FAILED;
   }

   return UT_Storage::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function GetJoin()
  {
   $d=new Data("unit_testing_pluginConfigs", "view", "all");
   $s=new System_Storage_mysql($d);

   //   SELECT nr,
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr, plugin,
   //     FROM unit_testing_pluginConfigs
   //LEFT JOIN pluginConfigs on nr=nr
   //    WHERE (true)
   $d->setShowFields("plugin");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?.*.?pluginConfigs.?\..?plugin.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" and \"plugin\" fields in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"pluginsConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr, display,
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   $d->setShowFields("display");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?.*.?pluginConfigs.?\..?display.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" and \"display\" fields in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM unit_testing_pluginConfigs
   //LEFT JOIN pluginConfigs on nr=nr
   //    WHERE (plugin="unit_testing")
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("plugin", "unit_testing");
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"pluginsConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?plugin.?=.?unit_testing.?/", $s->sql))
   {
    $this->e->add("Can't find \"plugin=unit_testing\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM unit_testing_pluginConfigs
   //    WHERE (display="list_units")
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("display", "list_units");
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?display.?=.?list_units.?/", $s->sql))
   {
    $this->e->add("Can't find \"display=list_units\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM unit_testing_pluginConfigs
   //LEFT JOIN pluginConfigs on nr=nr
   //    WHERE (true)
   //    ORDER BY plugin DESC
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder("plugin-");

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"pluginsConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?plugin.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"plugin DESC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   //    ORDER BY display DESC
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder("display-");

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?display.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"display DESC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT COUNT(*),
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder("display-");

   $s->e->clear();
   $s->get("count"); //count

   if(!preg_match("/SELECT COUNT\(\*\) AS .?show_count.?/", $s->sql))
   {
    $this->e->add("Unable to find \"count\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function GetItem()
  {
   $d=new Data("categories", "view", "item");
   $s=new System_Storage_mysql($d);

   //   SELECT nr,
   //     FROM categories
   //    WHERE (false)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?false.?/", $s->sql))
   {
    $this->e->add("Can't find \"false\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (nr=1)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("nr", 1);
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?nr.?=.?1.?/", $s->sql))
   {
    $this->e->add("Can't find \"nr=1\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (nr=1 OR nr=2)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("nr", 1);
   $d->setValue("nr", 2, 1);
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?nr.?=.?1.?.*OR.*\.?nr.?=.?2.?/", $s->sql))
   {
    $this->e->add("Can't find \"nr=1 OR nr=2\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function GetList()
  {
   $d=new Data("categories", "view", "list");
   $s=new System_Storage_mysql($d);

   //   SELECT nr,
   //     FROM categories
   //    WHERE (false)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?false.?/", $s->sql))
   {
    $this->e->add("Can't find \"false\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (catnr=1)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("catnr", 1);
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?catnr.?=.?1.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr=1\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (catnr=1 OR catnr=2)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("catnr", 1);
   $d->setValue("catnr", 2, 1);
   $d->sortOrder();

   $s->e->clear();
   $s->get();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?catnr.?=.?1.?.*OR.*\.?catnr.?=.?2.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr=1 OR catnr=2\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   return UT_OK;
  }

 }

?>
