<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/storage.xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["plugins"]."unit_testing/ut/storage.php");
 require_once($_path["include"]."storage/xml.php");
 require_once($_path["include"]."data.php");

 class UT_Storage_XML extends UT_Storage
 {
  function Constructor()
  {
   $d=new Data();
   $object=new System_Storage_xml($d, false);

   if(!is_object($object)||get_class($object)!="System_Storage_xml")
   {
    $this->e->add("Constructor doesn't return a System_Storage_xml object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $d=Data::cache();
   $orig=new System_Storage_xml($d);
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("System_Storage_xml object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->type="different";
   if($orig->type==$copy->type)
   {
    $this->e->add("System_Storage_xml object copy also modified original \"type\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("System_Storage_xml object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("System_Storage_xml object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("System_Storage_xml object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("System_Storage_xml object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->data->format=Array();
   if($orig->data->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_xml object copy didn't modify original \"Data\" object", "error");
    return UT_FAILED;
   }

   if($d->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_xml object copy didn't modify referenced \"Data\" object", "error");
    return UT_FAILED;
   }

   $copy->cache=Array();
   if($orig->cache!=$copy->cache)
   {
    $this->e->add("System_Storage_xml object copy didn't modify original \"cache\" reference", "error");
    return UT_FAILED;
   }

   return UT_Storage::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function _init()
  {
   global $_path;

   $this->_db=$_path["project"]["plugins"]."unit_testing/misc/test.xml";

   @unlink($this->_db);

   $d=new Data($this->_module, "database", "create", Array("db" => "xml://".$this->_db, "skipCheckTables" => true));

   if(!$d->submit())
    return false;

   return true;
  }

  function GetAggregation()
  {
   if(!$this->_init())
    return UT_FAILED;

   $d=new Data($this->_module, "view", "all", Array("db" => "xml://".$this->_db));
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "average");
   $d->setValue("aggregation_method", "around");
   $d->setValue("aggregation_interval", 1);

   $s=new System_Storage_xml($d);

   if($s->get())
   {
    $this->e->add("Getting aggregated data should fail, not support in class", "error");
    @unlink($this->_db);
    return UT_FAILED;
   }

   @unlink($this->_db);
   return UT_OK;
  }

 }

?>
